################################################################################
#  Copyright (C) 2006  Travis Shirk <travis@pobox.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id$
################################################################################
import sys, os

import utils
from i18n import _

from info import APP_NAME
from info import APP_VERSION
from info import APP_MAINTAINER

MESK_DIR = os.path.expandvars('${HOME}/.%s' % APP_NAME.lower())
DEFAULT_PLAYLISTS_DIR = "%s/playlists" % (MESK_DIR)
DEFAULT_PLUGINS_DIR = "%s/plugins" % (MESK_DIR)

# Config sections
CONFIG_MAIN     = 'mesk'
CONFIG_UI       = 'ui'
CONFIG_AUDIO    = 'audio'
CONFIG_PLAYLIST = 'playlist'

# GST sinks
GST_ALSA      = 'alsasink'
GST_OSS       = 'osssink'
GST_ESD       = 'esdsink'
GST_GCONF     = 'gconfaudiosink'
GST_AUTO_SINK = 'autoaudiosink'

DEFAULT_PLAYLIST_NAME = _('Playlist')

# Program options
# section: {opt_name: [type, default, help_string]}
options = {
    CONFIG_MAIN: {
    'version': [str, APP_VERSION, _('Application version')],
    'playlist_dir': [str, DEFAULT_PLAYLISTS_DIR,
                     _('The directory containing playlists.')],
    'active_playlist': [str, '',
                  _('The foreground playlist that has audio control.')],
    'playlists': [str, DEFAULT_PLAYLIST_NAME,
                  _('A list of playlists to open on startup.')],
    'log_level': [str, 'INFO',
                  _('Logging level. May be CRITICAL, ERROR, WARNING, INFO, '
                    'VERBOSE, or DEBUG')],
    'confirm_quit': [bool, True,
                     _('Show confirmation dialog before quitting the '
                       'application.')],
    'plugins_dir': [str, DEFAULT_PLUGINS_DIR,
                    _('The directory containing user installed plugins')],
    'plugins': [str, '', _('A list of plugins (separated by \';\') to activate '
                           'on startup.')],
    },

    CONFIG_UI: {
    'compact_state': [bool, False, _('Start in compact view mode')],
    'window_hide_on_close': [bool, True,
                             _('If True, the main window is hidded instead of '
                               'closed when the window close button is '
                               'clicked.')],
    'main_window_width': [int, 435, _('Main window width.')],
    'main_window_height': [int, 560, _('Main window height.')],
    'main_window_pos_x': [int, -1, _('Main window x origin.')],
    'main_window_pos_y': [int, -1, _('Main window y origin.')],
    'compact_main_window_pos_x': [int, -1,
                                  _('Main window x origin in compact mode.')],
    'compact_main_window_pos_y': [int, -1,
                                  _('Main window y origin in compact mode.')],
    'show_tab_close_button': [bool, False,
                              _('Display a close button on each tab.')],
    'show_tip_window_on_startup': [bool, True,
                                   _('Show tips window each time Mesk is '
                                     'started.')],
    'last_tip_displayed': [int, -1,
                           _('The last tip displayed in the tips window.')],
    'auto_open_devices': [bool, True,
                          _('Automatically open devices (cdroms, etc.)')],

    },

    CONFIG_AUDIO: {
    'gst_sink': [str, GST_ALSA,
                 _("GStreamer output sink. May be %s (default), %s, '%s',"
                   "'%s', or '%s'") % (GST_AUTO_SINK, GST_GCONF, GST_ALSA,
                                       GST_OSS, GST_ESD)],
    'gst_delay': [int, 1000,
                  _('Number of milliseconds to pause between tracks')],
    'volume': [float, 1.0,
               _('Volume level is a value between 0.0 and 1.0')],
    'mixer_command': [str, 'gnome-volume-control',
                      _('The volume level mixing program. The PATH environment '
                        'variable is used to locate commands that do not '
                        'contain a path.')],
    },

    CONFIG_PLAYLIST + '.' + DEFAULT_PLAYLIST_NAME: {
    'uri': [str, '', _('Playlist (file) location')],
    },
}

# Load config object with module level access
from config import Config
config = Config(options)

# Make mesk dir if needed
if not os.path.isdir(MESK_DIR):
    os.mkdir(MESK_DIR, 0751)
    os.mkdir(DEFAULT_PLAYLISTS_DIR, 0751)
    os.mkdir(DEFAULT_PLUGINS_DIR, 0751)

import mesk.log

from mesk.utils import MeskException
