################################################################################
#  Copyright (C) 2006  Travis Shirk <travis@pobox.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id$
################################################################################
import mesk

class PluginInfo(dict):
    def __init__(self, name=None, desc=None, author=None, url=None,
                 copyright=None, clazz=None, xpm=None, display_name=None):
        '''Despite the default arguments all except xpm are required'''
        self['NAME'] = name
        self['DESCRIPTION'] = desc
        self['AUTHOR'] = author
        self['URL'] = url
        self['COPYRIGHT'] = copyright
        self['XPM'] = xpm or DEFAULT_PLUGIN_XPM
        self['CLASS'] = clazz
        self['DISPLAY_NAME'] = display_name

        # Ensure all required values were set
        for key in self:
            if not self[key]:
                raise KeyError('Missing value in PluginInfo for key \'%s\'' %
                               key)

class Plugin(object):
    __plugin_info = None

    # Plugin name
    name = property(lambda self: self.__plugin_info['NAME'])
    # Plugin description
    description = property(lambda self: self.__plugin_info['DESCRIPTION'])
    # Plugin author(s)
    author = property(lambda self: self.__plugin_info['AUTHOR'])
    # Plugin URL
    url = property(lambda self: self.__plugin_info['URL'])
    # Plugin copyright
    copyright = property(lambda self: self.__plugin_info['COPYRIGHT'])
    # Plugin image in XPM format
    xpm = property(lambda self: self.__plugin_info['XPM'])

    def __init__(self, plugin_info):
        self.__plugin_info = plugin_info
        # An logger for each specific plugin
        self.log = mesk.log.getLogger(self.name)
        # The configuration window
        self.config_glade = None
        self.config_window = None

    def shutdown(self):
        '''Called when the plugin is shutdown or deactivated'''

    def is_configurable(self):
        '''Return True if the plugin supports configuration.
           Defaults to False'''
        return False

    def get_config_widget(self, parent):
        '''Return the configuration widget.'''
        return None
    def config_ok(self):
        '''Called when the configuration Ok button is clicked'''
    def config_cancel(self):
        '''Called when the configuration Cancel button is clicked'''

# An XPM for plugins which don't want to supply their own 
DEFAULT_PLUGIN_XPM = [
"48 36 714 2",
"  	c None",
". 	c #000000",
"+ 	c #110605",
"@ 	c #120705",
"# 	c #150805",
"$ 	c #150705",
"% 	c #150704",
"& 	c #140603",
"* 	c #110502",
"= 	c #0F0503",
"- 	c #050202",
"; 	c #2C100C",
"> 	c #4A1D13",
", 	c #5D2317",
"' 	c #5F2115",
") 	c #5D1D10",
"! 	c #58190D",
"~ 	c #481409",
"{ 	c #270B05",
"] 	c #0E0502",
"^ 	c #3D170F",
"/ 	c #7A2E1F",
"( 	c #A83E2A",
"_ 	c #AD3C26",
": 	c #A9351D",
"< 	c #9C2D16",
"[ 	c #762110",
"} 	c #3B1008",
"| 	c #0B0301",
"1 	c #110907",
"2 	c #120B09",
"3 	c #120B0A",
"4 	c #130C0B",
"5 	c #100C0B",
"6 	c #56281E",
"7 	c #A04634",
"8 	c #DF5D44",
"9 	c #E8573C",
"0 	c #E2492C",
"a 	c #CE3B1E",
"b 	c #942913",
"c 	c #451309",
"d 	c #0C0302",
"e 	c #49160C",
"f 	c #542219",
"g 	c #613128",
"h 	c #673B31",
"i 	c #693F37",
"j 	c #69423B",
"k 	c #68463F",
"l 	c #935E53",
"m 	c #C07769",
"n 	c #E68977",
"o 	c #E77660",
"p 	c #E0583D",
"q 	c #CA3D21",
"r 	c #912712",
"s 	c #441309",
"t 	c #51170C",
"u 	c #672012",
"v 	c #7F3426",
"w 	c #9C4F41",
"x 	c #B36656",
"y 	c #B86F60",
"z 	c #B87468",
"A 	c #B9796E",
"B 	c #CC8F83",
"C 	c #E0A69B",
"D 	c #F0B3A8",
"E 	c #EC9482",
"F 	c #E2664E",
"G 	c #CA4025",
"H 	c #912711",
"I 	c #010101",
"J 	c #210E08",
"K 	c #39170F",
"L 	c #63271B",
"M 	c #7D3426",
"N 	c #9E4D3E",
"O 	c #C56F5E",
"P 	c #E98E7B",
"Q 	c #F09584",
"R 	c #F09887",
"S 	c #EF9B8A",
"T 	c #F3AFA2",
"U 	c #F6C6BB",
"V 	c #F7D1C9",
"W 	c #EEA89A",
"X 	c #E46F59",
"Y 	c #CB4227",
"Z 	c #912611",
"` 	c #431209",
" .	c #0D0503",
"..	c #050708",
"+.	c #0F1010",
"@.	c #151615",
"#.	c #161716",
"$.	c #161717",
"%.	c #171818",
"&.	c #191919",
"*.	c #141414",
"=.	c #040100",
"-.	c #100907",
";.	c #3E2722",
">.	c #7B4E45",
",.	c #A66B5F",
"'.	c #C28276",
").	c #D9998D",
"!.	c #EFAEA1",
"~.	c #F1A698",
"{.	c #EE9A8A",
"].	c #EC8D7A",
"^.	c #F0A496",
"/.	c #F4C0B4",
"(.	c #F7D0C7",
"_.	c #421108",
":.	c #120908",
"<.	c #1C1F1E",
"[.	c #3E423F",
"}.	c #585E5A",
"|.	c #5A605B",
"1.	c #5A605C",
"2.	c #606561",
"3.	c #666967",
"4.	c #656866",
"5.	c #4E4F4E",
"6.	c #272827",
"7.	c #070707",
"8.	c #0B0808",
"9.	c #40312F",
"0.	c #8F7069",
"a.	c #CC9F97",
"b.	c #E6B7AE",
"c.	c #EDC4BC",
"d.	c #F5CEC6",
"e.	c #F4BAAD",
"f.	c #EF9D8D",
"g.	c #EA7F6A",
"h.	c #EE9B8A",
"i.	c #F4BBAF",
"j.	c #F7CFC6",
"k.	c #421107",
"l.	c #170E0C",
"m.	c #343836",
"n.	c #6D746F",
"o.	c #9BA69D",
"p.	c #9EA9A1",
"q.	c #9FAAA2",
"r.	c #A9B1AA",
"s.	c #B3B9B4",
"t.	c #AEB3AF",
"u.	c #7B7F7C",
"v.	c #383938",
"w.	c #090909",
"x.	c #1A0804",
"y.	c #1A0602",
"z.	c #27110C",
"A.	c #583D37",
"B.	c #A6837C",
"C.	c #E3BCB5",
"D.	c #F9D5CD",
"E.	c #F9DCD6",
"F.	c #F9E0DC",
"G.	c #F5C3BA",
"H.	c #EF9F8E",
"I.	c #E97861",
"J.	c #EE9584",
"K.	c #F3B8AC",
"L.	c #421007",
"M.	c #170F0D",
"N.	c #363B39",
"O.	c #737A74",
"P.	c #A4AEA6",
"Q.	c #A7B2AA",
"R.	c #A6B2A9",
"S.	c #A7B3AA",
"T.	c #B2BAB4",
"U.	c #BDC3BF",
"V.	c #B7BBB8",
"W.	c #7F8280",
"X.	c #393939",
"Y.	c #040101",
"Z.	c #280B06",
"`.	c #4A1309",
" +	c #652214",
".+	c #8C4B3F",
"++	c #C08C81",
"@+	c #EBC0B7",
"#+	c #FAD7D0",
"$+	c #F9DDD8",
"%+	c #FAE1DD",
"&+	c #F5C4BB",
"*+	c #E97760",
"=+	c #EE9583",
"-+	c #130A09",
";+	c #222524",
">+	c #494E4A",
",+	c #676E69",
"'+	c #6A716B",
")+	c #6A706B",
"!+	c #6A716C",
"~+	c #707671",
"{+	c #777C78",
"]+	c #757876",
"^+	c #555756",
"/+	c #282928",
"(+	c #300B03",
"_+	c #711C0B",
":+	c #A4311A",
"<+	c #C55A44",
"[+	c #DD9284",
"}+	c #F1C1B7",
"|+	c #F9D6CF",
"1+	c #F9DDD7",
"2+	c #EFA08F",
"3+	c #EE9684",
"4+	c #0E0604",
"5+	c #0A0C0B",
"6+	c #181918",
"7+	c #222422",
"8+	c #232523",
"9+	c #222523",
"0+	c #252725",
"a+	c #272927",
"b+	c #262827",
"c+	c #1D1F1E",
"d+	c #0F0F0F",
"e+	c #030303",
"f+	c #2D2927",
"g+	c #242120",
"h+	c #262220",
"i+	c #252220",
"j+	c #252221",
"k+	c #262221",
"l+	c #272221",
"m+	c #282322",
"n+	c #2A2423",
"o+	c #573129",
"p+	c #964030",
"q+	c #C9543D",
"r+	c #E47661",
"s+	c #ECA394",
"t+	c #F5C8BE",
"u+	c #F8D8D2",
"v+	c #FAE0DB",
"w+	c #F4C0B6",
"x+	c #EE9887",
"y+	c #E86F57",
"z+	c #ED8F7D",
"A+	c #F3B6A9",
"B+	c #F7CFC7",
"C+	c #EEA99B",
"D+	c #000101",
"E+	c #020202",
"F+	c #030403",
"G+	c #040404",
"H+	c #6B625E",
"I+	c #6E6461",
"J+	c #706461",
"K+	c #706562",
"L+	c #716562",
"M+	c #726663",
"N+	c #746764",
"O+	c #766764",
"P+	c #776865",
"Q+	c #786865",
"R+	c #947169",
"S+	c #BC7B6E",
"T+	c #DE8877",
"U+	c #EE9E8E",
"V+	c #F2BAB0",
"W+	c #F7D3CB",
"X+	c #F9DCD7",
"Y+	c #F9DEDA",
"Z+	c #F3B9AD",
"`+	c #ED8C7A",
" @	c #E55E43",
".@	c #EB8470",
"+@	c #F2B1A4",
"@@	c #F6D0C8",
"#@	c #EEAA9D",
"$@	c #E4705A",
"%@	c #D6C6C2",
"&@	c #C7B8B3",
"*@	c #BCADA8",
"=@	c #BFAFAA",
"-@	c #C2B0AB",
";@	c #C4B2AD",
">@	c #C6B4AE",
",@	c #C8B6B0",
"'@	c #CBB7B2",
")@	c #D0BAB4",
"!@	c #D3BBB6",
"~@	c #D4BBB6",
"{@	c #DCBEB7",
"]@	c #E8C1B9",
"^@	c #F2C5BC",
"/@	c #F7CDC5",
"(@	c #FAE3DF",
"_@	c #FAE2DD",
":@	c #F2B2A5",
"<@	c #EB806A",
"[@	c #E24B2C",
"}@	c #EA7961",
"|@	c #F1AC9F",
"1@	c #F6D1CA",
"2@	c #EFACA0",
"3@	c #E4715B",
"4@	c #CDB0A8",
"5@	c #CFB1A9",
"6@	c #D4B4AB",
"7@	c #D7B5AC",
"8@	c #DAB5AD",
"9@	c #DDB7AE",
"0@	c #E1BAAF",
"a@	c #E4BBB1",
"b@	c #E9BDB2",
"c@	c #EEBFB4",
"d@	c #F3C1B6",
"e@	c #F4C1B6",
"f@	c #F3C0B6",
"g@	c #F2C0B6",
"h@	c #F2C0B5",
"i@	c #F3C7BE",
"j@	c #F6D0C9",
"k@	c #F8D7D0",
"l@	c #F8D6D0",
"m@	c #F8D3CC",
"n@	c #F6CCC3",
"o@	c #EFA394",
"p@	c #E7735C",
"q@	c #DE4121",
"r@	c #E67159",
"s@	c #EFA799",
"t@	c #F6CFC7",
"u@	c #EDAA9E",
"v@	c #E27059",
"w@	c #C84127",
"x@	c #8F2611",
"y@	c #BD8779",
"z@	c #BD8679",
"A@	c #C08679",
"B@	c #C48778",
"C@	c #C78778",
"D@	c #CB8778",
"E@	c #CF8878",
"F@	c #D48978",
"G@	c #DA8A78",
"H@	c #E18A78",
"I@	c #E88B78",
"J@	c #E78A78",
"K@	c #E48977",
"L@	c #E18977",
"M@	c #E08977",
"N@	c #E08876",
"O@	c #E08A79",
"P@	c #E59787",
"Q@	c #EDAA9D",
"R@	c #F3B8AD",
"S@	c #F4B6AA",
"T@	c #F2AEA1",
"U@	c #EA8E7B",
"V@	c #E1664E",
"W@	c #D83D1E",
"X@	c #E16C55",
"Y@	c #EDA293",
"Z@	c #F5C9C0",
"`@	c #EBA698",
" #	c #DE6D56",
".#	c #C34026",
"+#	c #8A2411",
"@#	c #401209",
"##	c #0B0302",
"$#	c #A85B49",
"%#	c #A95A48",
"&#	c #AC5947",
"*#	c #B15945",
"=#	c #B45944",
"-#	c #B95843",
";#	c #BE5841",
">#	c #C35740",
",#	c #CC563E",
"'#	c #D3563C",
")#	c #DB553A",
"!#	c #D95439",
"~#	c #D45239",
"{#	c #D05138",
"]#	c #CE5138",
"^#	c #CE5037",
"/#	c #CF543C",
"(#	c #D76650",
"_#	c #E38471",
":#	c #ED9A8A",
"<#	c #F09E8E",
"[#	c #EF9887",
"}#	c #ED917F",
"|#	c #E57762",
"1#	c #DB5A40",
"2#	c #D13A1C",
"3#	c #DD6951",
"4#	c #EB9D8D",
"5#	c #F4C3B8",
"6#	c #E9A192",
"7#	c #DA6A54",
"8#	c #BE3E25",
"9#	c #872310",
"0#	c #3F1208",
"a#	c #853928",
"b#	c #863927",
"c#	c #893825",
"d#	c #8C3724",
"e#	c #903622",
"f#	c #943520",
"g#	c #99341F",
"h#	c #9E331D",
"i#	c #A5311A",
"j#	c #AC3017",
"k#	c #B32F15",
"l#	c #B12E14",
"m#	c #AC2C14",
"n#	c #A82B13",
"o#	c #AF2D14",
"p#	c #B93014",
"q#	c #C3361A",
"r#	c #D04A30",
"s#	c #DD6851",
"t#	c #E8806B",
"u#	c #EC8671",
"v#	c #EB826D",
"w#	c #EA7D69",
"x#	c #E16952",
"y#	c #D65237",
"z#	c #CA381A",
"A#	c #D8654D",
"B#	c #E79786",
"C#	c #F2BBAF",
"D#	c #E79B8C",
"E#	c #D76750",
"F#	c #BC3D24",
"G#	c #852310",
"H#	c #3E1208",
"I#	c #492118",
"J#	c #4B2118",
"K#	c #4F2117",
"L#	c #502116",
"M#	c #532115",
"N#	c #552015",
"O#	c #581F14",
"P#	c #5A1F12",
"Q#	c #5E1E11",
"R#	c #621D0F",
"S#	c #671D0E",
"T#	c #651C0E",
"U#	c #621B0E",
"V#	c #611B0D",
"W#	c #7A2211",
"X#	c #9F2C15",
"Y#	c #BE371C",
"Z#	c #D1462B",
"`#	c #DC593F",
" $	c #E56950",
".$	c #E97159",
"+$	c #E9755E",
"@$	c #E97761",
"#$	c #DE654D",
"$$	c #D14E34",
"%$	c #C43619",
"&$	c #D35F49",
"*$	c #E48E7D",
"=$	c #F0B0A3",
"-$	c #E69383",
";$	c #D7634C",
">$	c #BC3C23",
",$	c #852410",
"'$	c #220F0C",
")$	c #1E0D0A",
"!$	c #1A0B08",
"~$	c #1B0B08",
"{$	c #1B0B07",
"]$	c #1C0A07",
"^$	c #1E0B06",
"/$	c #200A06",
"($	c #210A05",
"_$	c #230A05",
":$	c #230905",
"<$	c #220905",
"[$	c #4B140A",
"}$	c #852512",
"|$	c #B5331A",
"1$	c #CF3E23",
"2$	c #D8492D",
"3$	c #E15337",
"4$	c #E65D42",
"5$	c #E7684E",
"6$	c #E76F58",
"7$	c #DB5F47",
"8$	c #CD4A31",
"9$	c #BD3318",
"0$	c #CD5B44",
"a$	c #E08675",
"b$	c #EEA698",
"c$	c #E58B7A",
"d$	c #D75F49",
"e$	c #BC3B23",
"f$	c #852411",
"g$	c #2D0D06",
"h$	c #6A1E0E",
"i$	c #9B2B15",
"j$	c #B9351B",
"k$	c #CD3D20",
"l$	c #DD4628",
"m$	c #E45235",
"n$	c #E66046",
"o$	c #E76B52",
"p$	c #DA5D45",
"q$	c #CB492F",
"r$	c #BA3318",
"s$	c #CB5842",
"t$	c #DF8271",
"u$	c #EDA092",
"v$	c #E48776",
"w$	c #D65E46",
"x$	c #BC3B22",
"y$	c #3E1108",
"z$	c #0D0603",
"A$	c #252726",
"B$	c #292929",
"C$	c #202020",
"D$	c #010000",
"E$	c #020101",
"F$	c #050201",
"G$	c #240A05",
"H$	c #49140A",
"I$	c #8B2916",
"J$	c #B5381F",
"K$	c #D84629",
"L$	c #E55436",
"M$	c #E66147",
"N$	c #E76C53",
"O$	c #CB5843",
"P$	c #EDA193",
"Q$	c #D65E47",
"R$	c #3D1007",
"S$	c #120A08",
"T$	c #727773",
"U$	c #7A7D7A",
"V$	c #7A7B7A",
"W$	c #5B5D5C",
"X$	c #2B2C2C",
"Y$	c #040202",
"Z$	c #070302",
"`$	c #170704",
" %	c #360F07",
".%	c #601D10",
"+%	c #9F311C",
"@%	c #D14428",
"#%	c #E45336",
"$%	c #E56046",
"%%	c #DA5D44",
"&%	c #3C0F06",
"*%	c #160F0C",
"=%	c #B4BCB6",
"-%	c #C1C6C2",
";%	c #BCC0BE",
">%	c #858685",
",%	c #3B3C3B",
"'%	c #100503",
")%	c #110503",
"!%	c #1D0904",
"~%	c #47160C",
"{%	c #852917",
"]%	c #B63B22",
"^%	c #CE492F",
"/%	c #D9573E",
"(%	c #E46349",
"_%	c #D9573D",
":%	c #CA462C",
"<%	c #CA553F",
"[%	c #DD7C69",
"}%	c #EB9889",
"|%	c #E2816F",
"1%	c #D35A42",
"2%	c #B83920",
"3%	c #832310",
"4%	c #3B0F06",
"5%	c #150E0C",
"6%	c #ABB2AD",
"7%	c #B7BCB9",
"8%	c #B3B6B3",
"9%	c #7D7F7E",
"0%	c #373938",
"a%	c #3A120A",
"b%	c #651F10",
"c%	c #862A18",
"d%	c #A43722",
"e%	c #C3482F",
"f%	c #DE5539",
"g%	c #D74D31",
"h%	c #C84026",
"i%	c #B93318",
"j%	c #C95039",
"k%	c #DB725E",
"l%	c #E88A78",
"m%	c #DE7562",
"n%	c #CD533B",
"o%	c #B2371E",
"p%	c #7D2210",
"q%	c #390F07",
"r%	c #1C201E",
"s%	c #616562",
"t%	c #686B69",
"u%	c #676868",
"v%	c #4C4D4D",
"w%	c #242424",
"x%	c #060606",
"y%	c #190805",
"z%	c #260C06",
"A%	c #3F1209",
"B%	c #55190D",
"C%	c #7B2615",
"D%	c #AD3820",
"E%	c #DA4729",
"F%	c #D74326",
"G%	c #C83C1F",
"H%	c #B93319",
"I%	c #C84D34",
"J%	c #DA6852",
"K%	c #E67C67",
"L%	c #D96A55",
"M%	c #C74D35",
"N%	c #AB341C",
"O%	c #79210F",
"P%	c #380F08",
"Q%	c #0B0403",
"R%	c #050808",
"S%	c #191A19",
"T%	c #2C0D06",
"U%	c #3D1109",
"V%	c #5E1B0E",
"W%	c #8A2916",
"X%	c #AD351D",
"Y%	c #AA331B",
"Z%	c #9F2E18",
"`%	c #932813",
" &	c #AE412C",
".&	c #CC5B45",
"+&	c #E26F59",
"@&	c #D7604A",
"#&	c #C34730",
"$&	c #A6311B",
"%&	c #751F0F",
"&&	c #370F07",
"*&	c #0A0301",
"=&	c #280B05",
"-&	c #41130A",
";&	c #5B1B0E",
">&	c #6E2112",
",&	c #6B1F11",
"'&	c #661D0F",
")&	c #5E190C",
"!&	c #893222",
"~&	c #B84E39",
"{&	c #DD634A",
"]&	c #D4573E",
"^&	c #BF422A",
"/&	c #A22F19",
"(&	c #721F0F",
"_&	c #090301",
":&	c #140503",
"<&	c #200905",
"[&	c #270B06",
"}&	c #270A06",
"|&	c #260A05",
"1&	c #230904",
"2&	c #622317",
"3&	c #A6402C",
"4&	c #DD573B",
"5&	c #D64E33",
"6&	c #BE3E24",
"7&	c #A12E18",
"8&	c #701F0F",
"9&	c #340F07",
"0&	c #100402",
"a&	c #150503",
"b&	c #160603",
"c&	c #150603",
"d&	c #140502",
"e&	c #4C1B11",
"f&	c #833120",
"g&	c #AC412B",
"h&	c #A73B25",
"i&	c #942F1B",
"j&	c #7E2413",
"k&	c #5B190C",
"l&	c #2D0C06",
"m&	c #090302",
"n&	c #120503",
"o&	c #37130C",
"p&	c #561F15",
"q&	c #6A2719",
"r&	c #662316",
"s&	c #210905",
"t&	c #090201",
"u&	c #1A0805",
"v&	c #240C07",
"w&	c #2A0D08",
"x&	c #2A0C07",
"y&	c #250A05",
"z&	c #1E0804",
"A&	c #130503",
"B&	c #110402",
"C&	c #130402",
"D&	c #130502",
"E&	c #0D0402",
"                                                        . . . .                                 ",
"                                                        . . . . .                               ",
"                                                    + @ # $ % & * =                             ",
"                                                  - ; > , ' ) ! ~ { ]                           ",
"                                            . . . . ^ / ( _ : < [ } | .                         ",
"                                          1 2 3 4 5 6 7 8 9 0 a b c d . .                       ",
"                                      e f g h i j k l m n o p q r s d . . .                     ",
"                                    t u v w x y z A B C D E F G H s d . I I I I I I I I I       ",
"                                J K L M N O P Q R S T U V W X Y Z `  ...+.@.#.#.$.%.&.&.*.      ",
"                              =.-.;.>.,.'.).!.~.{.].^./.(.W X Y Z _.:.<.[.}.|.|.1.2.3.4.5.6.7.  ",
"                              . 8.9.0.a.b.c.d.e.f.g.h.i.j.W X Y Z k.l.m.n.o.p.p.q.r.s.t.u.v.w.  ",
"                            x.y.z.A.B.C.D.E.F.G.H.I.J.K.j.W X Y Z L.M.N.O.P.Q.R.S.T.U.V.W.X.w.. ",
"                  . . . . Y.Z.`. +.+++@+#+$+%+&+H.*+=+K.j.W X Y Z _.-+;+>+,+'+)+!+~+{+]+^+/+7.. ",
"      . . . . . . . . . . . (+_+:+<+[+}+|+1+%+&+2+I.3+K.j.W X Y Z ` 4+5+6+7+8+9+;+0+a+b+c+d+e+  ",
"    f+g+h+i+j+k+l+m+m+m+m+n+o+p+q+r+s+t+u+$+v+w+x+y+z+A+B+C+X Y Z s d D+E+e+F+F+F+F+G+G+e+I I   ",
"    H+I+J+K+L+M+N+O+P+Q+Q+Q+R+S+T+U+V+W+X+Y+$+Z+`+ @.@+@@@#@$@Y Z s d . . . . . . . . .         ",
"%@&@*@=@-@;@>@,@'@)@!@~@~@~@{@]@^@/@u+%+(@_@E.:@<@[@}@|@1@2@3@Y Z s d . .                       ",
"4@5@6@7@8@9@0@a@b@c@d@e@f@g@g@h@}+i@j@k@l@m@n@o@p@q@r@s@t@u@v@w@x@` d . .                       ",
"y@z@A@B@C@D@E@F@G@H@I@J@K@L@M@N@O@P@Q@R@i.S@T@U@V@W@X@Y@Z@`@ #.#+#@###. .                       ",
"$#%#&#*#=#-#;#>#,#'#)#!#~#{#]#^#/#(#_#:#<#[#}#|#1#2#3#4#5#6#7#8#9#0#| . .                       ",
"a#b#c#d#e#f#g#h#i#j#k#l#m#n#o#p#q#r#s#t#u#v#w#x#y#z#A#B#C#D#E#F#G#H#| . .                       ",
"I#J#K#L#M#N#O#P#Q#R#S#T#U#V#W#X#Y#Z#`# $.$+$@$#$$$%$&$*$=$-$;$>$,$H#| . . . . . . . .           ",
"'$)$!$~${$]$^$^$/$($_$:$<$<$[$}$|$1$2$3$4$5$6$7$8$9$0$a$b$c$d$e$f$H#| D+E+e+F+F+F+F+G+G+e+      ",
"    . . . . . . . . . . . . g$h$i$j$k$l$m$n$o$p$q$r$s$t$u$v$w$x$f$y$z$5+6+7+8+9+;+A$/+B$C$      ",
"            . . . . . D$E$F$G$H$S#I$J$K$L$M$N$p$q$r$O$t$P$v$Q$x$f$R$S$;+>+,+'+)+!+T$U$V$W$X$7.  ",
"                        Y$Z$`$G$ %.%+%@%#%$%o$%%q$r$s$t$u$v$w$x$f$&%*%N.O.P.Q.R.S.=%-%;%>%,%w.  ",
"                            '%)%!%~%{%]%^%/%(%_%:%r$<%[%}%|%1%2%3%4%5%m.n.o.p.p.q.6%7%8%9%0%w.. ",
"                              )%x.a%b%c%d%e%f%g%h%i%j%k%l%m%n%o%p%q%-.r%[.}.|.|.1.s%t%u%v%w%x%  ",
"                                y%z%A%B%C%D%E%F%G%H%I%J%K%L%M%N%O%P%Q%R%+.@.#.#.$.6+S%S%*.w.    ",
"                                    T%U%V%W%X%Y%Z%`% &.&+&@&#&$&%&&&*&. I I I I I I I I I .     ",
"                                      =&-&;&>&,&'&)&!&~&{&]&^&/&(& %_&. . . . . . . . .         ",
"                                        :&<&[&}&|&1&2&3&4&5&6&7&8&9&_&. .                       ",
"                                          0&a&b&c&d&e&f&g&h&i&j&k&l&m&. D$                      ",
"                                              n&)%'%o&p&q&r&) t y$s&t&I                         ",
"                                                    u&v&w&x&Z.y&z&A&m&                          ",
"                                                      B&C&C&A&D&0&E&                            "
]

